# -*- coding: utf-8 -*-

import os
import json
import logging
from itertools import combinations

from ydl_ai_recommender.src.utils import get_data_path
from ydl_ai_recommender.src.utils.log import create_logger


class BasicUserSimilarity():

    def __init__(self) -> None:
        self.local_file_dir = get_data_path()
        self.logger = create_logger(__name__, 'basic_user_similarity')


    def compute_similarity(self):
        user_counselor_index = {}
        with open(os.path.join(self.local_file_dir, 'user_doctor_index.json'), encoding='utf-8') as f:
            user_counselor_index = json.load(f)

        user_like_set = {}
        counselor_user_index = {}

        for user, counselors in user_counselor_index.items():
            user_like_set[user] = set([c[0] for c in counselors])

            for [counselor, _, _] in counselors:
                if counselor not in counselor_user_index:
                    counselor_user_index[counselor] = []
                counselor_user_index[counselor].append(user)

        # 两个用户与同一个咨询师有订单，就认为两个用户相似
        self.logger.info('开始构建用户之间相似性关系')
        relations = {}
        user_index = {}
        for users in counselor_user_index.values():
            for [_u1, _u2] in combinations(users, 2):
                u1, u2 = min(_u1, _u2), max(_u1, _u2)
                key = '{}_{}'.format(u1, u2)
                if key in relations:
                    continue
                sim = len(user_like_set[u1] & user_like_set[u2]) / (len(user_like_set[u1]) * len(user_like_set[u2]))
                relations[key] = sim

                if u1 not in user_index:
                    user_index[u1] = {}
                if u2 not in user_index:
                    user_index[u2] = {}
                
                if u2 not in user_index[u1]:
                    user_index[u1][u2] = sim
                if u1 not in user_index[u2]:
                    user_index[u2][u1] = sim

        user_counselor_index = {}
        

        self.logger.info('用户相似性关系构建完成，共有 %s 对关系', len(relations))
        with open(os.path.join(self.local_file_dir, 'user_similarity.json'), 'w', encoding='utf-8') as f:
            json.dump(relations, f, ensure_ascii=False, indent=2)

    def recall(self, user, N=10, top_k=10):
        pass
                
        
bs = BasicUserSimilarity()
bs.compute_similarity()