# -*- coding: utf-8 -*-

import configparser

import pymysql

from ydl_ai_recommender.src.utils.log import create_logger


class MySQLClient():

    def __init__(self, host, port, user, password) -> None:
        self.logger = create_logger(__name__, 'mysql_client.log', is_rotating=True)
        try:
            self.connection = pymysql.connect(
                host=host,
                port=port,
                user=user,
                password=password,
                charset='utf8mb4',
                cursorclass=pymysql.cursors.DictCursor
            )
        except Exception as e:
            self.logger.error('数据库连接失败', exc_info=True)
            raise e
        self.cursor = self.connection.cursor()
        self.logger.info('数据库连接成功')


    def query(self, sql):
        # sql += ' limit 1000'
        self.logger.debug('begin execute sql: %s', sql)
        row_count = self.cursor.execute(sql)
        data = self.cursor.fetchall()
        self.logger.debug('fetch row count: %s', row_count)
        return row_count, data


    def __del__(self):
        try:
            self.cursor.close()
            self.connection.close()
            self.logger.info('dataset disconnected')
        except Exception as e:
            self.logger.error('销毁 MySQLClient 失败', exc_info=True)
            print(e)


    @classmethod
    def create_from_config_file(cls, config_file, section='ADB'):
        config = configparser.RawConfigParser()
        config.read(config_file)
        return cls(
            config.get(section, 'host'),
            config.getint(section, 'port'),
            config.get(section, 'user'),
            config.get(section, 'password')
        )